/*

  /-------------------------------------------------\
  |                                                 |
  |                                                 |
  |     /=====\     /=====\     |\          /|      |
  |    /       \   /       \    | \        / |      |
  |    |           |            |  \      /  |      |
  |    |           |            |   \    /   |      |
  |    \       /   \       /    |    \  /    |      |
  |     \=====/     \=====/     |     \/     |      |
  |                                                 |
  |                                                 |
  |           =====>>>>> 2002 <<<<=====             |
  |                                                 |
  \-------------------------------------------------/

  ===================================================

  Name:       CCM (input.C)
  
  Written by: Damian J. J. Farnell

  Date:       Wed Jul 17 12:03:05 BST 2002

  Purpose:    Inputs the CCM script data

  Changes:
      - cout replaced by printf, little improvements (Joerg Sep2005)

  ===================================================

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// #include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <string.h>

#include "Constants.h"
#include "Externals.h"
#include "FunctionPrototypes.h"

#if (PARALLEL_FLAG>0)
#include<mpi.h>
#endif

FILE *script;
FILE *escript;

// function prototypes.

void  moveAlongHome(FILE *fp);
double  readInFloat(FILE *fp);
int   readInInteger(FILE *fp);

void   definePointsInBT(void);

void input(int run_option, char *ccmfile) 
{
  
  int i, j ,k, bonds, expect, max_s2;
  
  char tempPath[100]="";        // initialized with "" to avoid
  char input_filename[100]="";  //   warnings from valgrind
  char temp[100]="";            // "jump depend from uninitialized variable"
  char configurations2[100]="";
  char expectations_input_filename[100]="";

  float dummyFloat;

  Control.file=1;    // tells program to save CCM equations to file.

  strcpy(input_filename,"");  // initial empty string
  strcpy(expectations_input_filename,"");  // initial empty string
  // old: always add prefix ../Data/
  // new: only   add prefix ../Data/ (for compatibility) if no path is given
  if (strchr(ccmfile,'/')==NULL) strcat(input_filename,"../Data/");
  if (strchr(ccmfile,'/')==NULL) strcat(expectations_input_filename,"../Data/");
  strcat(input_filename,ccmfile);
  strcat(expectations_input_filename,ccmfile);
  if (myid==0)
  printf("myid=%d name=%s runoption=%d\n",myid,input_filename,run_option);
  strcat(expectations_input_filename,"_exp");

  if (myid==0)
  printf("/-------------------------------------------------\\\n"
	 "|                                                 |\n"
	 "|                                                 |\n"
	 "|     /=====\\     /=====\\     |\\          /|      |\n"
	 "|    /           /            | \\        / |      |\n"
	 "|    |           |            |  \\      /  |      |\n"
	 "|    |           |            |   \\    /   |      |\n"
	 "|    \\           \\            |    \\  /    |      |\n"
	 "|     \\=====/     \\=====/     |     \\/     |      |\n"
	 "|                                                 |\n"
	 "|                                                 |\n"
	 "|           =====>>>>> 2002 <<<<=====             |\n"
	 "|                                                 |\n"
	 "\\-------------------------------------------------/\n"
	 "\n"
	 "===================================================\n"
	 "Name:       ccm.C\n"
	 "@Copyright: Damian J. J. Farnell\n"
	 "Date:       Wed Jul 17 12:03:05 BST 2002\n"
	 "Purpose:    Determine and solve CCM LSUBm equations.\n"
	 "===================================================\n\n");

  script=fopen(input_filename,"r");
  if (!script) {
    printf("input: Cannot Open %20s ...\nExiting!!\n\n",input_filename);
    exit(1);
  }

  // read in `control' parameters first:

  if (Control.output > 0 && myid ==0)
    printf("=======================================\n"
	   "Control Parameters:\n");

  moveAlongHome(script); Control.output=readInInteger(script);   // output level= ...

  if (Control.output > 0 && myid ==0)
    printf("Output (0-non to >6 verbose) =%d\n",Control.output);

  moveAlongHome(script); Control.greatExpectations=readInInteger(script);

  if (Control.output > 0 && myid ==0)
    printf("Find Generalised Expectations (0-No : 1-Yes) =%d\n",Control.greatExpectations);

  if (myid==0 && Control.output > 0 && Control.greatExpectations > 0)
    printf("myid=%d expectations name=%s\n",myid,expectations_input_filename);

  if (Control.greatExpectations > 0) {
    escript=fopen(expectations_input_filename,"r");
    if (!escript) {
      printf("input expectations input file: Cannot Open %20s ...\nExiting!!\n\n",expectations_input_filename);
      exit(1);
    }
  }

  moveAlongHome(script); fscanf(script,"%80s",tempPath);  // work directory= ...
  moveAlongHome(script); fscanf(script,"%80s",temp);      // output file= ...
  
  sprintf(Control.name,"%s%s",tempPath,temp);
#if (PARALLEL_FLAG>0)
  // Oct05: myid-prefix changed to suffix
  sprintf(Control.name,"%s%s_id%d",tempPath,temp,myid);
#endif

  if (myid==0)
  printf("Simulation Filename = %s : myid == %d\n",Control.name,myid);

  // read in parameters for solve.C
  
  // now read in the `solve' variables

  moveAlongHome(script); minimum=readInFloat(script);
  moveAlongHome(script); maximum=readInFloat(script);
  moveAlongHome(script); increment=readInFloat(script);
  if (Control.output > 0 && myid ==0) {
    printf("Minimum delta=  %20.10f\n",minimum);
    printf("Maximum delta=  %20.10f\n",maximum);
    printf("Increment delta=%30.20f\n",increment);
  }

  // next two variables are minimum and maximum y values
  // used in plot.tcl but not in ccm C program.

  moveAlongHome(script); dummyFloat=readInFloat(script);
  moveAlongHome(script); dummyFloat=readInFloat(script);

  // read in highest level parameters first:

  if (Control.output > 0 && myid ==0)
    printf("\nGeneral Parameters:\n");
  
  // dimension

  moveAlongHome(script); Params.dim=readInInteger(script);
  if (Control.output > 0 && myid ==0)
    printf("Dimension=%d\n",Params.dim);

  // spin number

  moveAlongHome(script); dummyFloat = readInFloat(script);
  // Joerg: We store the twofold of Spin quantum number to have an integer
  //        this is faster and makes the program more compact
  // ToDo: its the maximum of spins in the unit cell and can be
  //       automatically calculated (see below), so its obsolete to set here
  Params.s2 = (int) (2*dummyFloat + 0.5);
  if (Control.output > 0 && myid ==0)
    printf("Maximum spin quantum number=%f\n",Params.s2 / 2.0);

  initDivisors();  // store 0! .. (2s+1)! into array

  // Bounding Triangle

  // dummyInt=0;

  moveAlongHome(script); Params.btLength=readInInteger(script);
  if (Control.output > 0 && myid ==0)
    printf("BT Length=%d\n",Params.btLength);  // Joerg: bounding triangle
  // JS: length of triangle block >= m ? 
  
  // SUBn-m: n= spin flips
  // SUBn-m: m= max. geom. distance
  moveAlongHome(script); Params.n=readInInteger(script);
  moveAlongHome(script); Params.m=readInInteger(script);
  if (Control.output > -1 && myid ==0) // essential debug info
    printf("SUBn-m: n=%d m=%d\n", Params.n, Params.m);
  
  if (Params.btLength < Params.m) {
     if (!myid) printf("Warning input: bounding < m, set bounding = m\n");
     Params.btLength = Params.m; // Joerg Oct05, set bounding triangle to m
  }

  moveAlongHome(script); Params.bonds=readInInteger(script);
  if (Control.output > 0 && myid ==0)
    printf("Bonds: m=%d\n",Params.bonds);

  if (Params.bonds > MAX_BONDS) { // defined in Externals.h
    printf("Error input: Bonds > MAX_BONDS\n");
    exit(1);
  }

  moveAlongHome(script); restrictionClusters=readInInteger(script);
  if (Control.output > 0 && myid ==0)
    printf("Restriction on Clusters: Type=%d\n",restrictionClusters);
  
  moveAlongHome(script); solutionStrategy=readInInteger(script);
  if (PARALLEL_FLAG>0 && solutionStrategy == 1) {
    if (myid==0)
      printf("WARNING: solutionStrategy 1 (Newton-Raphson) dont work for MPI\n"
            "switching to solutionStrategy = 0 (Direct Iteration)\n");
    // old code exit here, but we make life easier (JS Apr08)
    solutionStrategy=0;
  }
  if (Control.output > 0 && myid ==0)
    if (solutionStrategy == 1)
      printf("Solve CCM equations using Newton-Raphson method\n");
    else
      printf("Solve CCM equations using Direct Iteration method\n");
  
  if (Control.output > 0 && myid ==0)
    printf("=======================================\n"
	   "The Unit Cell:\n");
  
  // number of different elements of a symmetric multidimensional matrix
  if (Params.dim == 1) 
    max_number_cells_BT=Params.btLength;
  else if (Params.dim == 2)
    max_number_cells_BT=Params.btLength*(Params.btLength+1)/2;
  else {
    // Joerg Sep05, ToDo: use dimension independent formular
    max_number_cells_BT=Params.btLength*(Params.btLength+1)/2
                                       *(Params.btLength+2)/3;
  }

  if (Control.output > 1 && myid ==0) 
    printf("Dimension=%d\ns=%5.3f\nm=%d\nB.T.Length=%d\n"
	   "#Sites in B.T.=%d\n\n",
	   Params.dim,
	   Params.s2 / 2.0,
	   Params.m,
	   Params.btLength,
	   max_number_cells_BT);

  // 1 <= max_number_cells_BT <= MAX_BT  (Joerg Sep05)

  if ( max_number_cells_BT > MAX_BT) {
    printf("Number of Cells in BT is greater than MAX_BT...\n"
	  "Exiting!\n\n");
    exit(1);
  }

  // probably a good idea to keep 
  // Params.btLength >= Params.m/UnitCell.number+1;

  for(i=0;i<MAX_UNIT_CELL;i++) {
    UnitCell.x[i]=0.0;
    UnitCell.y[i]=0.0;
    UnitCell.z[i]=0.0;
    UnitCell.s2[i]=1; // 2*s
  }

  moveAlongHome(script); fscanf(script,"%d",&UnitCell.number);
  if (Control.output > 0 && myid ==0)
    printf("Number of sites in Unit Cell = %d\n",UnitCell.number);
  if (Control.output > 0 && myid ==0)
    printf("Positions & Spin Quantum Number:\n");

  if (UnitCell.number>=MAX_UNIT_CELL)
    { printf("Error: UnitCell.number>=MAX_UNIT_CELL\nExit\n"); exit(1); }

  // read in all sites of the unit cell
  for(max_s2=1,i=1;i<=UnitCell.number;i++) {
    
    UnitCell.x[i]=readInFloat(script);
    UnitCell.y[i]=readInFloat(script);
    UnitCell.z[i]=readInFloat(script);
    UnitCell.s2[i]=(int)(2*fabs(readInFloat(script))+0.5);
    //    moveAlongHome(script);
    if (max_s2 < UnitCell.s2[i])
        max_s2 = UnitCell.s2[i];

  }

  // check that spin length doesnt exceed maxspinlength (Joerg 2005)
  if (max_s2 != Params.s2) {
     if (!myid) {
       printf("\nWARNING input: (max_spin=%3.1f) !=  (spin=%3.1f),"
              " set spin automatically to max_spin\n",
              0.5*max_s2, 0.5*Params.s2);
       printf("--- Do you forgot to set spin length in the unit cell? ---\n");
     }
     Params.s2 = max_s2;
     initDivisors();  // store 0! .. (2s+1)! into array
     // exit(1); // We dont need to worry, Joerg
  }

  for(i=1;i<=UnitCell.number;i++) {
    if (Control.output > 0 && myid ==0)
      printf("Site= %2d : position = (% f,% f,% f) : spin = %f\n",i,
	     UnitCell.x[i],
	     UnitCell.y[i],
	     UnitCell.z[i],
	     0.5*UnitCell.s2[i]);
  }

  if (Control.output > 0 && myid ==0)
    printf("=======================================\n"
	   "The Bravais Lattice:\n");

  // define the bravais lattice 

  moveAlongHome(script);
  BravaisLattice.a[1]=readInFloat(script);
  BravaisLattice.a[2]=readInFloat(script);
  BravaisLattice.a[3]=readInFloat(script);
  BravaisLattice.b[1]=readInFloat(script);
  BravaisLattice.b[2]=readInFloat(script);
  BravaisLattice.b[3]=readInFloat(script);
  BravaisLattice.c[1]=readInFloat(script);
  BravaisLattice.c[2]=readInFloat(script);
  BravaisLattice.c[3]=readInFloat(script);
  
  if (Control.output > 0 && myid ==0) {
    printf("Bravais Lattice Vector A=(% 6.4f,% 6.4f,% 6.4f)\n",
	   BravaisLattice.a[1],BravaisLattice.a[2],BravaisLattice.a[3]);
    printf("Bravais Lattice Vector B=(% 6.4f,% 6.4f,% 6.4f)\n",
	   BravaisLattice.b[1],BravaisLattice.b[2],BravaisLattice.b[3]);
    printf("Bravais Lattice Vector C=(% 6.4f,% 6.4f,% 6.4f)\n",
	   BravaisLattice.c[1],BravaisLattice.c[2],BravaisLattice.c[3]);
  }

  if (Control.output > 0 && myid ==0)
    printf("=======================================\n"
	   "Nearest-Neighbours to sites in Unit Cell:\n");
  

  for(i=1;i<=UnitCell.number;i++) {
    moveAlongHome(script);
    LatticeNeighbours.number[i]=readInInteger(script); 
    for(j=1;j<=LatticeNeighbours.number[i];j++) {
	  LatticeNeighbours.x[i][j]=readInInteger(script);
	  LatticeNeighbours.y[i][j]=readInInteger(script);
	  LatticeNeighbours.z[i][j]=readInInteger(script);
	  LatticeNeighbours.cell[i][j]=readInInteger(script);
    }
  }

  if (Control.output > 0 && myid ==0) {
    for(i=1;i<=UnitCell.number;i++) {
      printf("Number of nearest-neighbours for site %d = %d : \n",
	    i, LatticeNeighbours.number[i]);
      for(j=1;j<=LatticeNeighbours.number[i];j++) {
        // ToDo: Neighbour = Bravais translation? wrong for ferriLC (JS06) 
	printf(" %d : Bravais translation (x y z site) = (% d % d % d % d)\n",
	       j, LatticeNeighbours.x[i][j],
                  LatticeNeighbours.y[i][j],
	          LatticeNeighbours.z[i][j],
                  LatticeNeighbours.cell[i][j]);
      }
    }
    printf("=======================================\n"
	   "The symmetry operations:\n");
  }
  moveAlongHome(script);

  LatticeSymmetries.number=readInInteger(script); 

  if (LatticeSymmetries.number<1) {
    printf("input: we need at least the one symmetry operator (identity)\n"
           "  to get findDescriptorOfCluster running\nexit!\n");
    exit(1);
  }
  // just to make sure set to zero initially.

  for(i=1;i<=LatticeSymmetries.number;i++){
    LatticeSymmetries.x[i]=0.0;
    LatticeSymmetries.y[i]=0.0;
    LatticeSymmetries.z[i]=0.0;
    for(j=1;j<=3;j++){
      for(k=1;k<=3;k++){
	LatticeSymmetries.R[i][j][k]=0.0;
      }
    }
  }

  for(i=1;i<=LatticeSymmetries.number;i++) {
    for(j=1;j<=3;j++){
      for(k=1;k<=3;k++){
	LatticeSymmetries.R[i][j][k]=readInFloat(script);
      }
      if      (j==1) LatticeSymmetries.x[i]=readInFloat(script);
      else if (j==2) LatticeSymmetries.y[i]=readInFloat(script);
      else           LatticeSymmetries.z[i]=readInFloat(script);
    }
  }

  if (Control.output > 0 && myid ==0) {
    for(i=1;i<=LatticeSymmetries.number;i++) {
      printf("Symmetry Operation = %d\nRotation matrix : Prim. Transl. \n",i);
      for(j=1;j<=3;j++){
	for(k=1;k<=3;k++){
	  printf("% 5.3f ", LatticeSymmetries.R[i][j][k]);
	}
	if (j==1)      printf(" : % 5.3f\n",LatticeSymmetries.x[i]);
	else if (j==2) printf(" : % 5.3f\n",LatticeSymmetries.y[i]);
	else           printf(" : % 5.3f\n",LatticeSymmetries.z[i]);
      }
    } 
  }

  // now determine the Hamiltonian. The `types' of term
  // first...
  
  if (Control.output > 0 && myid ==0)
    printf("=======================================\n"
	   "The Hamiltonian:\n");
  
  for(bonds=1;bonds<=Params.bonds;bonds++) {
    for(i=0;i<MAX_UNIT_CELL;i++) {
      for(j=0;j<MAX_NUM_NEIGHBOURS;j++) {
	for(k=0;k<15;k++) {  // array defined as [200], why? Joerg05
	  Hamiltonian_types[bonds][k].k_to_m_number[i]=0;
	  Hamiltonian_types[bonds][k].m_to_k_number[i]=0;
	  Hamiltonian_types[bonds][k].k_to_m[i][j][0]=0; // x
	  Hamiltonian_types[bonds][k].k_to_m[i][j][1]=0; // y
	  Hamiltonian_types[bonds][k].k_to_m[i][j][2]=0; // z
	  Hamiltonian_types[bonds][k].k_to_m[i][j][3]=0; // cell
	  Hamiltonian_types[bonds][k].m_to_k[i][j][0]=0; // x
	  Hamiltonian_types[bonds][k].m_to_k[i][j][1]=0; // y
	  Hamiltonian_types[bonds][k].m_to_k[i][j][2]=0; // z
	  Hamiltonian_types[bonds][k].m_to_k[i][j][3]=0; // cell
	  Hamiltonian_types[bonds][k].coefficient[i][j]=0.0;
	}
      }
    }
  }

  moveAlongHome(script);
  directional=readInInteger(script);

  if (myid==0)
    printf("Directional?=%d  (1=directional, 0=non-directional bonds)\n",
           directional);

  if ((!myid) && directional) // see ToDo below
     printf("\nWarning: Overwriting forward coefficient. "
            "If that causes error, report to JS!\n"); // Jun06

  for(bonds=1;bonds<=Params.bonds;bonds++) {
    for(i=1;i<=UnitCell.number;i++) {
      for(k=1;k<14;k++) {    /* 1..13 */

	moveAlongHome(script);
	Hamiltonian_types[bonds][k].k_to_m_number[i]=readInInteger(script);
	
	if (k<=9) {  /* 1..9 are bonds */

          /* read in forward directions */
          for(j=1;j<=Hamiltonian_types[bonds][k].k_to_m_number[i];j++) {
            Hamiltonian_types[bonds][k].k_to_m[i][j][0]=readInInteger(script);
            Hamiltonian_types[bonds][k].k_to_m[i][j][1]=readInInteger(script);
            Hamiltonian_types[bonds][k].k_to_m[i][j][2]=readInInteger(script);
            Hamiltonian_types[bonds][k].k_to_m[i][j][3]=readInInteger(script);
            Hamiltonian_types[bonds][k].coefficient[i][j]=readInFloat(script);
          }

	  if (directional) { /* two directions, read in backwards direction */

	    moveAlongHome(script);
	    Hamiltonian_types[bonds][k].m_to_k_number[i]=readInInteger(script);

	    for(j=1;j<=Hamiltonian_types[bonds][k].m_to_k_number[i];j++) {
	      Hamiltonian_types[bonds][k].m_to_k[i][j][0]=readInInteger(script); // x
	      Hamiltonian_types[bonds][k].m_to_k[i][j][1]=readInInteger(script); // y
	      Hamiltonian_types[bonds][k].m_to_k[i][j][2]=readInInteger(script); // z
	      Hamiltonian_types[bonds][k].m_to_k[i][j][3]=readInInteger(script); // cell
	      Hamiltonian_types[bonds][k].coefficient[i][j]=readInFloat(script);
              /* ToDo: overwiting same coefficient? */
	    }

	  } else {    /* one direction, copy forwards to backwards direction */

	    Hamiltonian_types[bonds][k].m_to_k_number[i]=
	      Hamiltonian_types[bonds][k].k_to_m_number[i];

	    for(j=1;j<=Hamiltonian_types[bonds][k].k_to_m_number[i];j++) {
	      Hamiltonian_types[bonds][k].m_to_k[i][j][0]=Hamiltonian_types[bonds][k].k_to_m[i][j][0];
	      Hamiltonian_types[bonds][k].m_to_k[i][j][1]=Hamiltonian_types[bonds][k].k_to_m[i][j][1];
	      Hamiltonian_types[bonds][k].m_to_k[i][j][2]=Hamiltonian_types[bonds][k].k_to_m[i][j][2];
	      Hamiltonian_types[bonds][k].m_to_k[i][j][3]=Hamiltonian_types[bonds][k].k_to_m[i][j][3];	      

	    }
	  }
	} else {  /* 10..13 are single site terms -> 56,57-59,60-62,63 */
	  Hamiltonian_types[bonds][k].coefficient[i][1]=readInFloat(script);
	}
      }
    }
  }

  /* set flag to evaluate if we have nonzero coefficient (gt 0.01) */
  for(bonds=1;bonds<=Params.bonds;bonds++) {
    for(k=1;k<14;k++) {
      Hamiltonian_types[bonds][k].evaluate=0;
      for(i=1;i<=UnitCell.number;i++) {
	for(j=1;j<=Hamiltonian_types[bonds][k].k_to_m_number[i];j++) {
//        until 0605 only coefficients bigger than 0.01 are used
//        if (fabs(Hamiltonian_types[bonds][k].coefficient[i][j]) > 0.01)
	  if (Hamiltonian_types[bonds][k].coefficient[i][j] != 0.0) {
	    Hamiltonian_types[bonds][k].evaluate=1;
	    break;
	  }
	}
	if (Hamiltonian_types[bonds][k].evaluate) break;
      }
    }
  }

  // now print out this information:

    
  if (Control.output > 0 && myid ==0) {
    const char *tdescr[14]={"none",
      "s^z s^z",
      "s^z s^+", "s^z s^-", "s^+ s^z", "s^- s^z",
      "s^+ s^-", "s^- s^+", "s^+ s^+", "s^- s^-",
      "s^z", "s^-", "(s^z)^2", "s^+"};  // Oct05 Joerg: correct???
    for(bonds=1;bonds<=Params.bonds;bonds++) {
      for(k=1;k<14;k++) {
        if (!Hamiltonian_types[bonds][k].evaluate) continue; // Oct05
	if (Control.output>0)
          printf("\n");  // space before next type of bonds
	if (Control.output>0) 
	  printf("Bond = %d : Type = %2d : Evaluate =%d : %s\n"
		 ,bonds,k, Hamiltonian_types[bonds][k].evaluate, tdescr[k]);
	if (Control.output>0) {
	  if (k > 9) {
	    for(i=1;i<=UnitCell.number;i++) {
	      printf("Site = %d : Single Site Term\n",i);
	      printf("     (x y z uc) = (% d % d % d % d) : Coeff. = % f\n",
		     Hamiltonian_types[bonds][k].k_to_m[i][1][0],
		     Hamiltonian_types[bonds][k].k_to_m[i][1][1],
		     Hamiltonian_types[bonds][k].k_to_m[i][1][2],
		     Hamiltonian_types[bonds][k].k_to_m[i][1][3],
		     Hamiltonian_types[bonds][k].coefficient[i][1]);
	    }
	  } else {

	    for(i=1;i<=UnitCell.number;i++) {

              printf("Site = %d\n",i);
              if (directional) printf("Site = %d : k to m\n",i);
              for(j=1;j<=Hamiltonian_types[bonds][k].k_to_m_number[i];j++) {
                printf(" %d : (x y z uc) = (% d % d % d % d) "
                       ": Coeff. = % f\n",j,
                       Hamiltonian_types[bonds][k].k_to_m[i][j][0],
                       Hamiltonian_types[bonds][k].k_to_m[i][j][1],
                       Hamiltonian_types[bonds][k].k_to_m[i][j][2],
                       Hamiltonian_types[bonds][k].k_to_m[i][j][3],
                       Hamiltonian_types[bonds][k].coefficient[i][j]);
              }

	      if (directional) { /* directional */
		printf("Site = %d : m to k\n",i);
		for(j=1;j<=Hamiltonian_types[bonds][k].m_to_k_number[i];j++) {
		  printf(" %d : (x y z uc) = (% d % d % d % d) "
                         ": Coeff. = % f\n",j,
			 Hamiltonian_types[bonds][k].m_to_k[i][j][0], // x
			 Hamiltonian_types[bonds][k].m_to_k[i][j][1], // y
			 Hamiltonian_types[bonds][k].m_to_k[i][j][2], // z
			 Hamiltonian_types[bonds][k].m_to_k[i][j][3], // cell
			 Hamiltonian_types[bonds][k].coefficient[i][j]);
		}
	      }
	    }
	  }
	}
      }
    }
  }

  // now input the information for the Hamiltonian proper for each term
  // in the similarity transformed version of H.

  // expand H_types(terms=13) to H(terms=62)
  //   1 -> 1,2,3,4,5
  //   2 -> 6,7
  //   3 -> 8-13,46-48
  //   4 -> ...
  // ToDo: its simpler to have an array [1..63] -> 1..13
  //       and use it backwards

  for(bonds=1;bonds<=Params.bonds;bonds++) {
    // Oct05: new code is much simplier (one dimensional array)
    for (int term=1; term<=62; term++) {
      k = term_13[term];
      if (Hamiltonian_types[bonds][k].evaluate) {
        // k = term_13[term62]
        Hamiltonian[bonds][term].evaluate=1;

        if (k > 9) {
          for(i=1;i<=UnitCell.number;i++) {
            Hamiltonian[bonds][term].k_to_m[i][1][0]=Hamiltonian_types[bonds][k].k_to_m[i][1][0];
            Hamiltonian[bonds][term].k_to_m[i][1][1]=Hamiltonian_types[bonds][k].k_to_m[i][1][1];
            Hamiltonian[bonds][term].k_to_m[i][1][2]=Hamiltonian_types[bonds][k].k_to_m[i][1][2];
            Hamiltonian[bonds][term].k_to_m[i][1][3]=Hamiltonian_types[bonds][k].k_to_m[i][1][3];
            Hamiltonian[bonds][term].coefficient[i][1]=Hamiltonian_types[bonds][k].coefficient[i][1];
          }
        } else {

          for(i=1;i<=UnitCell.number;i++) {
            
// ToDo:    Hamiltonian[bonds][term] = Hamiltonian_types[bonds][k];

            Hamiltonian[bonds][term].k_to_m_number[i]=
              Hamiltonian_types[bonds][k].k_to_m_number[i];
            
            for(j=1;j<=Hamiltonian_types[bonds][k].k_to_m_number[i];j++) {

              Hamiltonian[bonds][term].k_to_m[i][j][0]=Hamiltonian_types[bonds][k].k_to_m[i][j][0];
              Hamiltonian[bonds][term].k_to_m[i][j][1]=Hamiltonian_types[bonds][k].k_to_m[i][j][1];
              Hamiltonian[bonds][term].k_to_m[i][j][2]=Hamiltonian_types[bonds][k].k_to_m[i][j][2];
              Hamiltonian[bonds][term].k_to_m[i][j][3]=Hamiltonian_types[bonds][k].k_to_m[i][j][3];
              Hamiltonian[bonds][term].coefficient[i][j]=Hamiltonian_types[bonds][k].coefficient[i][j];

            }
            
            Hamiltonian[bonds][term].m_to_k_number[i]=
              Hamiltonian_types[bonds][k].m_to_k_number[i];
            
            for(j=1;j<=Hamiltonian_types[bonds][k].m_to_k_number[i];j++) {

              Hamiltonian[bonds][term].m_to_k[i][j][0]=Hamiltonian_types[bonds][k].m_to_k[i][j][0];
              Hamiltonian[bonds][term].m_to_k[i][j][1]=Hamiltonian_types[bonds][k].m_to_k[i][j][1];
              Hamiltonian[bonds][term].m_to_k[i][j][2]=Hamiltonian_types[bonds][k].m_to_k[i][j][2];
              Hamiltonian[bonds][term].m_to_k[i][j][3]=Hamiltonian_types[bonds][k].m_to_k[i][j][3];
              Hamiltonian[bonds][term].coefficient[i][j]=Hamiltonian_types[bonds][k].coefficient[i][j];

            }
          }
        }
      }
    }
  }

  if (Control.output>3 && myid ==0) {

    printf("\nprint nonzero of 62 terms (expanded from 13 terms) now:\n");
    for(bonds=1;bonds<=Params.bonds;bonds++) {
      printf("\n");

      for(k=1;k<63;k++) {

	printf("Bond = %d : Type = %d : Evaluate =%d\n",bonds,k,
	       Hamiltonian[bonds][k].evaluate);      

	if (k > 53) {

	  for(i=1;i<=UnitCell.number;i++) {
	    printf("Site = %d : # Single Site Term : ",i);
	    printf("(x y z uc) = (% d % d % d % d) : Coeff. = % f\n",
		   Hamiltonian[bonds][k].k_to_m[i][1][0],
		   Hamiltonian[bonds][k].k_to_m[i][1][1],
		   Hamiltonian[bonds][k].k_to_m[i][1][2],
		   Hamiltonian[bonds][k].k_to_m[i][1][3],
		   Hamiltonian[bonds][k].coefficient[i][1]);
	  }

	} else {

	  for(i=1;i<=UnitCell.number;i++) {

	    if (directional) {

	      printf("Site = %d : k to m\n",i);
	      for(j=1;j<=Hamiltonian[bonds][k].k_to_m_number[i];j++) {
                printf("#%d : (x y z uc) = (% d % d % d % d) "
                       ": Coeff. = % f\n",j,
		       Hamiltonian[bonds][k].k_to_m[i][j][0],
		       Hamiltonian[bonds][k].k_to_m[i][j][1],
		       Hamiltonian[bonds][k].k_to_m[i][j][2],
		       Hamiltonian[bonds][k].k_to_m[i][j][3],
		       Hamiltonian[bonds][k].coefficient[i][j]);
	      }
	      printf("Site = %d : m to k\n",i);
	      for(j=1;j<=Hamiltonian[bonds][k].m_to_k_number[i];j++) {
                printf("#%d : (x y z uc) = (% d % d % d % d) "
                       ": Coeff. = % f\n",j,
		       Hamiltonian[bonds][k].m_to_k[i][j][0],
		       Hamiltonian[bonds][k].m_to_k[i][j][1],
		       Hamiltonian[bonds][k].m_to_k[i][j][2],
		       Hamiltonian[bonds][k].m_to_k[i][j][3],
		       Hamiltonian[bonds][k].coefficient[i][j]);
	      }

	    } else {

	      printf("Site = %d\n",i);
	      for(j=1;j<=Hamiltonian[bonds][k].k_to_m_number[i];j++) {
                printf("#%d : (x y z uc) = (% d % d % d % d) "
                       ": Coeff. = % f\n",j,
		       Hamiltonian[bonds][k].k_to_m[i][j][0],
		       Hamiltonian[bonds][k].k_to_m[i][j][1],
		       Hamiltonian[bonds][k].k_to_m[i][j][2],
		       Hamiltonian[bonds][k].k_to_m[i][j][3],
		       Hamiltonian[bonds][k].coefficient[i][j]);
	      }

	    }
	  }
	}
      }
    }
  }

  fclose(script);
  
  // 5-5-08: Additional code to implement generalised expectation values 
  if (Control.greatExpectations > 0) {
  
    // first: the number of expectation values
    moveAlongHome(escript); Params.nExpect=readInInteger(escript);   // output level= ...
    if (Control.output > 0 && myid ==0)  printf("\n\nNumber of Expectation Values = %d \n",Params.nExpect);

    for(expect=1;expect<=Params.nExpect;expect++) {
      for(i=1;i<=UnitCell.number;i++) {
        for(k=1;k<14;k++) { /* 1..13 */

     	   moveAlongHome(escript);
	       expectation_types[expect][k].k_to_m_number[i]=readInInteger(escript);
	
        	if (k<=9) {  /* 1..9 are bonds */

              /* read in forward directions */
			  for(j=1;j<=expectation_types[expect][k].k_to_m_number[i];j++) {
                 expectation_types[expect][k].k_to_m[i][j][0]=readInInteger(escript);
                 expectation_types[expect][k].k_to_m[i][j][1]=readInInteger(escript);
                 expectation_types[expect][k].k_to_m[i][j][2]=readInInteger(escript);
                 expectation_types[expect][k].k_to_m[i][j][3]=readInInteger(escript);
                 expectation_types[expect][k].coefficient[i][j]=readInFloat(escript);
              }

       	  if (directional) { /* two directions, read in backwards direction */

	         moveAlongHome(escript);
	         expectation_types[expect][k].m_to_k_number[i]=readInInteger(escript);

			for(j=1;j<=expectation_types[expect][k].m_to_k_number[i];j++) {
	           expectation_types[expect][k].m_to_k[i][j][0]=readInInteger(escript); // x
	           expectation_types[expect][k].m_to_k[i][j][1]=readInInteger(escript); // y
	           expectation_types[expect][k].m_to_k[i][j][2]=readInInteger(escript); // z
	           expectation_types[expect][k].m_to_k[i][j][3]=readInInteger(escript); // cell
	           expectation_types[expect][k].coefficient[i][j]=readInFloat(escript);
              /* ToDo: overwiting same coefficient? */
	        }

    	  } else {    /* one direction, copy forwards to backwards direction */

	        expectation_types[expect][k].m_to_k_number[i]=
	          expectation_types[expect][k].k_to_m_number[i];

     	    for(j=1;j<=expectation_types[expect][k].k_to_m_number[i];j++) {
	           expectation_types[expect][k].m_to_k[i][j][0]=expectation_types[expect][k].k_to_m[i][j][0];
	           expectation_types[expect][k].m_to_k[i][j][1]=expectation_types[expect][k].k_to_m[i][j][1];
	           expectation_types[expect][k].m_to_k[i][j][2]=expectation_types[expect][k].k_to_m[i][j][2];
	           expectation_types[expect][k].m_to_k[i][j][3]=expectation_types[expect][k].k_to_m[i][j][3];	      
    	    }
		 }
	  } else {  /* 10..13 are single site terms -> 56,57-59,60-62,63 */
	      expectation_types[expect][k].coefficient[i][1]=readInFloat(escript);
     	}
      }
    }
  }
  
  for(expect=1;expect<=Params.nExpect;expect++) 
      for(i=1;i<=UnitCell.number;i++) 
        for(k=1;k<14;k++) 
		  printf("(%d,%d,%d) : %d\n",expect,i,k,expectation_types[expect][k].k_to_m_number[i]);


  /* set flag to evaluate if we have nonzero coefficient (gt 0.01) */
  for(expect=1;expect<=Params.nExpect;expect++) {
    for(k=1;k<14;k++) {
      expectation_types[bonds][k].evaluate=0;
	  }
   }
	  
  for(expect=1;expect<=Params.nExpect;expect++) {
    for(k=1;k<14;k++) {
      for(i=1;i<=UnitCell.number;i++) {
/*    	for(j=1;j<=expectation_types[expect][k].k_to_m_number[i];j++) {

        if (fabs(Hamiltonian_types[bonds][k].coefficient[i][j]) != 0.0)
     printf("%d %d %d %d : %f\n",expect,k,i,j,expectation_types[expect][k].coefficient[i][j]);
	  if (expectation_types[expect][k].coefficient[i][j] != 0.0) {
	    expectation_types[expect][k].evaluate=1;
	    break;
	  }
	 } 
	if (expectation_types[expect][k].evaluate) break;
	}
*/

     if ((k> 9) && (fabs(expectation_types[expect][k].coefficient[i][j]) > 0.0001)) {
	      expectation_types[expect][k].evaluate=1;
	 }
     if ((k<10) && (expectation_types[expect][k].k_to_m_number[i] > 0)) {
	    expectation_types[expect][k].evaluate=1;
//	    break;
	  }
	  
	}
	
//	if (expectation_types[expect][k].evaluate) break;
    }
  }
	
 // now print out this information:

/*   for(expect=1;expect<=Params.nExpect;expect++) 
    for(k=1;k<14;k++) 
	  printf("%d %d : %d\n",expect,k,expectation_types[expect][k].evaluate);
*/

    
  if (Control.output > 0 && myid ==0) {
    const char *tdescr[14]={"none",
      "s^z s^z",
      "s^z s^+", "s^z s^-", "s^+ s^z", "s^- s^z",
      "s^+ s^-", "s^- s^+", "s^+ s^+", "s^- s^-",
      "s^z", "s^-", "(s^z)^2", "s^+"};  // Oct05 Joerg: correct???
    for(expect=1;expect<=Params.nExpect;expect++) {
      for(k=1;k<14;k++) {
        if (!expectation_types[expect][k].evaluate) continue; // Oct05
	if (Control.output>0)
          printf("\n");  // space before next type of expect
	if (Control.output>0) 
	  printf("Expect = %d : Type = %2d : Evaluate =%d : %s\n"
		 ,expect,k, expectation_types[expect][k].evaluate, tdescr[k]);
	if (Control.output>0) {
	  if (k > 9) {
	    for(i=1;i<=UnitCell.number;i++) {
	      printf("Site = %d : Single Site Term\n",i);
	      printf("     (x y z uc) = (% d % d % d % d) : Coeff. = % f\n",
		     expectation_types[expect][k].k_to_m[i][1][0],
		     expectation_types[expect][k].k_to_m[i][1][1],
		     expectation_types[expect][k].k_to_m[i][1][2],
		     expectation_types[expect][k].k_to_m[i][1][3],
		     expectation_types[expect][k].coefficient[i][1]);
	    }
	  } else {

	    for(i=1;i<=UnitCell.number;i++) {

              printf("Site = %d\n",i);
              if (directional) printf("Site = %d : k to m\n",i);
              for(j=1;j<=expectation_types[expect][k].k_to_m_number[i];j++) {
                printf(" %d : (x y z uc) = (% d % d % d % d) "
                       ": Coeff. = % f\n",j,
                       expectation_types[expect][k].k_to_m[i][j][0],
                       expectation_types[expect][k].k_to_m[i][j][1],
                       expectation_types[expect][k].k_to_m[i][j][2],
                       expectation_types[expect][k].k_to_m[i][j][3],
                       expectation_types[expect][k].coefficient[i][j]);
              }

	      if (directional) { /* directional */
		printf("Site = %d : m to k\n",i);
		for(j=1;j<=expectation_types[expect][k].m_to_k_number[i];j++) {
		  printf(" %d : (x y z uc) = (% d % d % d % d) "
                         ": Coeff. = % f\n",j,
			 expectation_types[expect][k].m_to_k[i][j][0], // x
			 expectation_types[expect][k].m_to_k[i][j][1], // y
			 expectation_types[expect][k].m_to_k[i][j][2], // z
			 expectation_types[expect][k].m_to_k[i][j][3], // cell
			 expectation_types[expect][k].coefficient[i][j]);
		}
	      }
	    }
	  }
	}
      }
    }
  }

	
 for(expect=1;expect<=Params.nExpect;expect++) {
    // Oct05: new code is much simplier (one dimensional array)
    for (int term=1; term<=62; term++) {
      k = term_13[term];
      if (expectation_types[expect][k].evaluate) {
        // k = term_13[term62]
        expectation[expect][term].evaluate=1;

        if (k > 9) {
          for(i=1;i<=UnitCell.number;i++) {
            expectation[expect][term].k_to_m[i][1][0]=expectation_types[expect][k].k_to_m[i][1][0];
            expectation[expect][term].k_to_m[i][1][1]=expectation_types[expect][k].k_to_m[i][1][1];
            expectation[expect][term].k_to_m[i][1][2]=expectation_types[expect][k].k_to_m[i][1][2];
            expectation[expect][term].k_to_m[i][1][3]=expectation_types[expect][k].k_to_m[i][1][3];
            expectation[expect][term].coefficient[i][1]=expectation_types[expect][k].coefficient[i][1];
          }
        } else {

          for(i=1;i<=UnitCell.number;i++) {
            
// ToDo:    expectation[expect][term] = expectation_types[expect][k];

            expectation[expect][term].k_to_m_number[i]=
              expectation_types[expect][k].k_to_m_number[i];
            
            for(j=1;j<=expectation_types[expect][k].k_to_m_number[i];j++) {

              expectation[expect][term].k_to_m[i][j][0]=expectation_types[expect][k].k_to_m[i][j][0];
              expectation[expect][term].k_to_m[i][j][1]=expectation_types[expect][k].k_to_m[i][j][1];
              expectation[expect][term].k_to_m[i][j][2]=expectation_types[expect][k].k_to_m[i][j][2];
              expectation[expect][term].k_to_m[i][j][3]=expectation_types[expect][k].k_to_m[i][j][3];
              expectation[expect][term].coefficient[i][j]=expectation_types[expect][k].coefficient[i][j];

            }
            
            expectation[expect][term].m_to_k_number[i]=
              expectation_types[expect][k].m_to_k_number[i];
            
            for(j=1;j<=expectation_types[expect][k].m_to_k_number[i];j++) {

              expectation[expect][term].m_to_k[i][j][0]=expectation_types[expect][k].m_to_k[i][j][0];
              expectation[expect][term].m_to_k[i][j][1]=expectation_types[expect][k].m_to_k[i][j][1];
              expectation[expect][term].m_to_k[i][j][2]=expectation_types[expect][k].m_to_k[i][j][2];
              expectation[expect][term].m_to_k[i][j][3]=expectation_types[expect][k].m_to_k[i][j][3];
              expectation[expect][term].coefficient[i][j]=expectation_types[expect][k].coefficient[i][j];

            }
          }
        }
      }
    }
  }

  if (Control.output>3 && myid ==0) {

    printf("\nprint nonzero of 62 terms (expanded from 13 terms) now:\n");
    for(expect=1;expect<=Params.nExpect;expect++) {
      printf("\n");

      for(k=1;k<63;k++) {

	printf("Bond = %d : Type = %d : Evaluate =%d\n",expect,k,
	       expectation[expect][k].evaluate);      

	if (k > 53) {

	  for(i=1;i<=UnitCell.number;i++) {
	    printf("Site = %d : # Single Site Term : ",i);
	    printf("(x y z uc) = (% d % d % d % d) : Coeff. = % f\n",
		   expectation[expect][k].k_to_m[i][1][0],
		   expectation[expect][k].k_to_m[i][1][1],
		   expectation[expect][k].k_to_m[i][1][2],
		   expectation[expect][k].k_to_m[i][1][3],
		   expectation[expect][k].coefficient[i][1]);
	  }

	} else {

	  for(i=1;i<=UnitCell.number;i++) {

	    if (directional) {

	      printf("Site = %d : k to m\n",i);
	      for(j=1;j<=expectation[expect][k].k_to_m_number[i];j++) {
                printf("#%d : (x y z uc) = (% d % d % d % d) "
                       ": Coeff. = % f\n",j,
		       expectation[expect][k].k_to_m[i][j][0],
		       expectation[expect][k].k_to_m[i][j][1],
		       expectation[expect][k].k_to_m[i][j][2],
		       expectation[expect][k].k_to_m[i][j][3],
		       expectation[expect][k].coefficient[i][j]);
	      }
	      printf("Site = %d : m to k\n",i);
	      for(j=1;j<=expectation[expect][k].m_to_k_number[i];j++) {
                printf("#%d : (x y z uc) = (% d % d % d % d) "
                       ": Coeff. = % f\n",j,
		       expectation[expect][k].m_to_k[i][j][0],
		       expectation[expect][k].m_to_k[i][j][1],
		       expectation[expect][k].m_to_k[i][j][2],
		       expectation[expect][k].m_to_k[i][j][3],
		       expectation[expect][k].coefficient[i][j]);
	      }

	    } else {

	      printf("Site = %d\n",i);
	      for(j=1;j<=expectation[expect][k].k_to_m_number[i];j++) {
                printf("#%d : (x y z uc) = (% d % d % d % d) "
                       ": Coeff. = % f\n",j,
		       expectation[expect][k].k_to_m[i][j][0],
		       expectation[expect][k].k_to_m[i][j][1],
		       expectation[expect][k].k_to_m[i][j][2],
		       expectation[expect][k].k_to_m[i][j][3],
		       expectation[expect][k].coefficient[i][j]);
	      }

	    }
	  }
	}
      }
    }
  }

  
    }
  
  definePointsInBT();

  //  init x_array[],x_index[],cell_index[]
  //  defineGeneralisedPositions();  -> removed (not used, Joerg05)

  // not used at moment...

  sublatticeMagnetisation.number=UnitCell.number;
  for (i=1;i<=sublatticeMagnetisation.number;i++)
    sublatticeMagnetisation.cell[i]=1;
 
  // read in cluster configurations 
  
  if ((run_option&1) == 0) { // loading configs from file, else compute
    int nread;

    strcpy(configurations2,Control.name);
    strcat(configurations2,".configs");
    if (!myid) 
      printf("\nReading ConfigDescriptors(n*%d*(int=%d)) from file %s\n",
                      MAX_SPIN_LIST, (int)sizeof(int), configurations2);
    if (!myid) {  // JS Apr08: read only once and distribute per MPI
      fp1=fopen(configurations2,"r"); // $(Control.name).configs  
      if (!fp1) {
        printf("input: Cannot Open %s...\nExiting!!\n\n",configurations2);
        exit(1);
      }
      // pre060611: fread(&NumberConfigurations,sizeof(int),1,fp1);    
      for(i=1;i<=MAX_NUM_CONFIGURATIONS;i++) {

        nread=fread(spinConfigurations[i].descriptor,
               ((MAX_SPIN_LIST)*sizeof(int)), 1, fp1);
        if (nread!=1) {
           if (feof(fp1)) { i--; break; }
           printf("%d: ferror %d\nExit!\n",myid,ferror(fp1));
           exit(1);
        }
        if (Control.output > 2 && myid==0 && i<10){
          printf("Cluster %d:\n",i);
          printOutClusterFromDescriptor(spinConfigurations[i].descriptor);
        }
        if (feof(fp1)) break;
      }
      fclose(fp1);
    }
    NumberConfigurations = i;

    if (!myid) {
      printf("\nFundamental clusters = %d\n", NumberConfigurations);
    }
#if PARALLEL_FLAG>0
    int iret=0;
    if (Control.output>0 && !myid)
      printf(" calling MPI_Bcast(.configs, (int)(%d*%d)) ...\n",
             sizeof(struct spinConfigs), i); 
    iret= MPI_Bcast(&NumberConfigurations, 1, MPI_INT, 0, MPI_COMM_WORLD);
    if (iret!=MPI_SUCCESS) MPI_Abort(MPI_COMM_WORLD, 1);
    i = NumberConfigurations;
    iret= MPI_Bcast(spinConfigurations+1,
               sizeof(struct spinConfigs)*i, MPI_BYTE, 0, MPI_COMM_WORLD);
    if (iret!=MPI_SUCCESS) MPI_Abort(MPI_COMM_WORLD, 1);
#endif

  }
}


void moveAlongHome(FILE *fp)
{
  char dummy, buff[4000];
  int  count, bp=0, iscomment=0;

  /* 
     purpose of this routine is to scan along the file until
     an `=' sign is seen 
     Joerg Sep05: ignore out commented lines (// ... and # ...)
  */
  
  for(count=0;count<4000;count++) {
    fscanf(fp,"%c",&dummy);
    buff[bp]=dummy; bp++; buff[bp]=0;
    if (dummy == '\n' || dummy == '=') {
       if (bp) buff[bp-1]='\n';
       if (Control.output > 2 && myid == 0) printf("%% %s",buff);
       bp=0; buff[bp]=0;
    }
    if (bp>1 && buff[bp-1] == '/' && buff[bp-2]== '/') iscomment=1;
    if (dummy == '#')  iscomment=1;
    if (dummy == '\n') iscomment=0;
    if (dummy == '=' && !iscomment) break;
  }

  if (count >=4000) {
    printf("Error in reading CCM script: can't find delimiter in time!\n"
	   "Exiting!!\n\n");
    exit(1);
  }

}

int readInInteger(FILE *fp)
{
  int sum;
  fscanf(fp,"%d",&sum);
  return sum;
}
    
double readInFloat(FILE *fp)
{
  double sum;
  fscanf(fp,"%lf",&sum);
  return sum;
}

// called by input()
// initialize: lookUpTable[0..MAX_BT-1]
//                 used by [printOut,find]ClusterFromDescriptor
//             BTarray[0..Params.btLength-1][y][x]
//                 used by findConfigurationToDescriptor
// ToDo: estimate size and malloc lookUpTable, simplify (Joerg)
//   sample: btLength=4 D=2
//   (y,x)=(0,0),(0,1),(0,2),(0,3),
//         (1,0),(1,1),(1,2),
//         (2,0),(2,1),
//         (3,0)                   n*(n+1)/2 = 4*5/2 = 10 entries
// rewritten by Joerg Oct05         
void definePointsInBT(void)
{

  int i, x,y,z,count,mx,my,mz;
  
  if (Params.btLength > MAX_LSUBM) {
    printf("Params.btLength=%d > MAX_LSUBM\n", Params.btLength);
    exit(1);
  }

  if (Params.btLength < Params.m) {
    printf("Params.btLength=%d < Params.m\n", Params.btLength);
    exit(1);
  }

  for(z=0;z<Params.btLength;z++) 
  for(y=0;y<Params.btLength;y++) 
  for(x=0;x<Params.btLength;x++) BTarray[z][y][x]=-1;
  
  for(i=0;i<MAX_BT;i++) {  // default: 20000
    lookUpTable[i][1]=-1;
    lookUpTable[i][2]=-1;
    lookUpTable[i][3]=-1;
  }

  mx=my=mz=Params.btLength; // maximum x,y,z for 3D-case
  if (Params.dim < 3) mz=1; // max_z=0 for 2D (periodic 2D lattice)
  if (Params.dim < 2) my=1; // max_y=0 for 1D (periodic 1D lattice)
  if (Params.dim < 1) mx=1; // max_x=0 for 0D (finite cluster = UC)

  count=0; // site index, starting from 0 (Oct05)

  for (z=0;z<mz    ;z++) 
  for (y=0;y<my-z  ;y++) 
  for (x=0;x<mx-y-z;x++) { // (x,y,z) <=> idx, replacable by 2 functions ???
    if (count > MAX_BT) { printf("Error: MAX_BT to small\n"); exit(1); }
    BTarray[z][y][x]=count;     // len=10 -> old=385 new=220
    lookUpTable[count][1]=x;
    lookUpTable[count][2]=y;
    lookUpTable[count][3]=z;
    count++;                    //  2D: n*(n+1)/2, 3D: n*(n+1)*(n+2)/6
  }

  // added by Joerg Sep2005  cubic 10 -> 10*12^2/4=1440/4=385=10*11/2*7
  if (Control.output > 0 && myid==0)
   printf("definePointsInBT: length= %d sites= %d\n", Params.btLength, count);
}  
